%%% Econ 203C (Spring 2009)
%%% Problem Set II, Question 4
%%% Monte Carlo Simulation of distribution of 2SLS

clear all;              % clear all the variables
clc;                    % clear the screen

% Set primitive variables
R=5000;                 % number of Monte Carlo simulation
n=100;                  % sample size in each simulation
beta=0;                 % true parameter to be estimated
K=[1,10];               % number of instrumental variables
eta=[.05,1];            % strength of instruments
rho=[0,.95];            % endogeneity of regressor

% Initialize the sequence of random numbers
randn('seed',2032009);

% Initialize the estimates matrix
beta_est = [];

% Start iteration
for r=1:R
    % Initialize the intermediate estimates matrix
    beta_temp = [];
    
    for h=1:length(K)
        % Generate random matrix used through all the eta's and the rho's
        V = randn(n,2);             % standard normal random vector with var=I
        Z = randn(n,K(h));          % standard normal random vector with var=I
        
        for i=1:length(eta)
            for j=1:length(rho)
                % Form intermediate parameters
                Omega = [1,rho(j);rho(j),1];
                Pi = eta(i) * ones(K(h),1);

                % Data Generating Process
                U = chol(Omega);    % Cholesky decomposition of Omega=U'U
                u = V*U;            % normal random vector with mean=0, var=Omega=U'U
                z = Z;              % normal random vector with mean=0; var=I
                x = z*Pi + u(:,2);
                y = x*beta + u(:,1);
    
                % 2SLS Estimation
                beta_hat = inv(x'*z*inv(z'*z)*z'*x)*x'*z*inv(z'*z)*z'*y;
                
                % Stack estimates to temporary space
                beta_temp = [beta_temp,beta_hat];
            end
        end
    end
    
    % Stack estimates for later use
    beta_est = [beta_est;beta_temp];
end

fprintf('\n---------------||------------------- K=1 ---------------||----------------- K=10 ----------------\n');
fprintf('  (eta / rho)  ||   (.05/0)  (.05/.5)     (1/0)    (1/.5)   (.05/0)  (.05/.5)     (1/0)    (1/.5)\n');
fprintf('---------------||--------------------------------------------------------------------------------\n');
fprintf('   Mean bias   ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',mean(beta_est)-beta);
fprintf('  Median bias  ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',median(beta_est)-beta);
fprintf(' Standard Devn ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',std(beta_est));
fprintf('  R M S Error  ||%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f%10.4f\n',sqrt(mean((beta_est-beta).^2)));
fprintf('\n');

% Plot histogram of estimates
figure(1);

% Set histogram format
CN = 0;                 % case number
NB = 100;               % number of bins

for h=1:length(K)
    for i=1:length(eta)
        for j=1:length(rho)
            CN = CN + 1;            % increase case number
            subplot(4,2,CN);        % use Panel CN
            hist(beta_est(:,CN),NB);% plot estimates of Case CN
            T = sprintf('K=%d,eta=%1.2f,rho=%1.2f',K(h),eta(i),rho(j));
            title(T);               % display the case
        end
    end
end

% Plot histogram (rescaled) and approximated distribution
figure(2);

% Set histogram format
CN = 0;                 % case number
NB = 50;                % number of bins

for h=1:length(K)
    for i=1:length(eta)
        for j=1:length(rho)
            CN = CN + 1;            % increase case number
            subplot(4,2,CN);        % use Panel CN
            
            % Calculate Asymptotic variance and Approximated variance
            asym_var = 1 / (K(h)*eta(i)^2);
            approx_var = asym_var / n;
            
            % Convert estimates data into hisogram format
            bound = max(quantile(abs(beta_est(:,CN)),0.98), 2*sqrt(approx_var));
                                    % set bounds to drop outliers
            domain = linspace(-bound,bound,NB+2);
                                    % divide domain into NB+2 bins of equal length 
            [num,domain] = hist(beta_est(:,CN),domain);
                                    % use estimates of Case CN
            freq = num / (R * 2 * bound / (NB+2));
                                    % rescale by the area of histogram
            
            % Drop outliers from histogram
            domain = domain(2:NB+1);
            freq = freq(2:NB+1);

            % Calculate the pdf of the approximated distribution
            approx_pdf = 1/sqrt(2*pi*approx_var) * exp(- domain.^2 / (2*approx_var));
            
            % Plot histogram of estimates and approximated pdf
            plot(domain,freq,'b-',domain,approx_pdf,'r--');
            axis([-bound bound 0 1.2*max([freq,approx_pdf])]);
                                    % set axis scales
            T = sprintf('K=%d,eta=%1.2f,rho=%1.2f',K(h),eta(i),rho(j));
            title(T);               % display the case
        end
    end
end
legend('Estimates','Approximate');  % show legend

% Plot histogram and asymptotic distribution
figure(3);

% Set histogram format
CN = 0;                 % case number
NB = 50;                % number of bins

for h=1:length(K)
    for i=1:length(eta)
        for j=1:length(rho)
            CN = CN + 1;            % increase case number
            subplot(4,2,CN);        % use the panel CN
            
            % Calculate Asymptotic variance
            asym_var = 1 / (K(h)*eta(i)^2);
            approx_var = asym_var / n;
            
            % Convert estimates data into hisogram format
            bound = max(quantile(abs(beta_est(:,CN)),0.98), 2*sqrt(approx_var));
                                    % set bounds to drop outliers
            domain = linspace(-bound,bound,NB+2);
                                    % divide domain into NB+2 bins of equal length 
            [num,domain] = hist(beta_est(:,CN),domain);
                                    % use estimates of Case CN
            
            % Draw from the approximated distribution
            approx_draw = sqrt(approx_var) * randn(R,1);
            [approx_num,domain] = hist(approx_draw,domain);

            % Drop outliers from histogram
            domain = domain(2:NB+1);
            num = num(2:NB+1);
            approx_num = approx_num(2:NB+1);

            % Plot histogram of estimates and approximated draw
            plot(domain,num,'b-',domain,approx_num,'r--');
            axis([-bound bound 0 1.2*max([num,approx_num])]);
                                    % set axis scales
            T = sprintf('K=%d,eta=%1.2f,rho=%1.2f',K(h),eta(i),rho(j));
            title(T);               % display the case
        end
    end
end
legend('Estimates','Approximate');  % show legend
